<?php

namespace SuperRhino\Utils\Redis;

use SuperRhino\Utils\Snowflake\SnowflakeGenerator;

/**
 * redis锁
 */
class Lock {

    /**
     * 加锁
     * @param string $lockKey 锁名
     * @param int $ttl        milliseconds
     * @return string|bool 锁id
     */
    public static function lock(string $lockKey, int $ttl) {
        $redis = Redis::getRedis();
        $lockId = SnowflakeGenerator::generateStr();

        $luaScript = "return redis.call('SET', KEYS[1], ARGV[1], 'PX', ARGV[2], 'NX')";
        $result = $redis->eval($luaScript, ['lock_' . $lockKey, $lockId, $ttl], 1);
        if ($result) {
            return $lockId;
        }
        return false;
    }

    /**
     * 解锁
     * @param string $lockKey 锁名
     * @param string $lockId  锁id
     * @return bool
     */
    public static function unlock(string $lockKey, string $lockId) {
        $redis = Redis::getRedis();
        $luaScript = <<<LUA
local value = redis.call('get', KEYS[1])
if (value == ARGV[1]) then
	return redis.call('del', KEYS[1]);
end
return 0;
LUA;
        $result = $redis->eval($luaScript, ['lock_' . $lockKey, $lockId], 1);
        return (bool)$result;
    }

    /**
     * 重入锁(继锁)
     * @param string $lock
     * @param string $lockId
     * @param int $ttl
     * @return bool
     */
    public static function relock(string $lock, string $lockId, int $ttl) {
        $redis = Redis::getRedis();
        $luaScript = <<<LUA
local value = redis.call('get', KEYS[1])
if (value == ARGV[1]) then
    return redis.call('SET', KEYS[1], ARGV[1], 'PX', ARGV[2])
end
return 0;
LUA;
        $result = $redis->eval($luaScript, ['lock_' . $lock, $lockId, $ttl], 1);
        return (bool)$result;
    }
}